package org.eclipse.ece.messaging.ws;

import java.io.IOException;
import java.util.Formatter;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ThreadLocalRandom;
import java.util.concurrent.atomic.AtomicReference;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.jetty.websocket.api.Session;
import org.eclipse.jetty.websocket.api.WebSocketAdapter;

public class EventSocket extends WebSocketAdapter {
	
	private final Logger logger = Logger.getLogger("wsServerSocket");
	public static final String MESSAGE_TEMPLATE = "{ client:\"%s\", value: %d, timestamp:%o}";
	private AtomicReference<Session> session = new AtomicReference<Session>(null);
	private Timer timer = new Timer("msg-timer");
	private TimerTask task = new TimerTask() {
		
		@Override
		public void run() {
			if (session.get() != null) {
				try {
					if (session.get() != null && session.get().isOpen()) {
						long ts = System.currentTimeMillis();
						int value = ThreadLocalRandom.current().nextInt(0, 25);
						Formatter formatter = new Formatter();
						String content = formatter.format(MESSAGE_TEMPLATE, "ws-server", value, ts).toString();
						formatter.close();
						logger.info("Server send: " + content);
						session.get().getRemote().sendString(content);
					} else {
						logger.info("Session is closed");
					}
				} catch (IOException e) {
					logger.log(Level.SEVERE, "Exception during sending of message", e);
				}
			}
		}
	};
	
    @Override
    public void onWebSocketConnect(Session sess) {
        super.onWebSocketConnect(sess);
        session.set(sess);
        logger.info("Server socket connected: " + sess);
    }
    
    @Override
    public void onWebSocketText(String message) {
        super.onWebSocketText(message);
        logger.info("Received TEXT message: " + message);
        if (message.equalsIgnoreCase("start") && session != null) {
        	logger.info("Starting timer");
        	timer.schedule(task, 10l, 1500l);
        }
        if (message.equalsIgnoreCase("stop") && session != null) {
        	logger.info("Stopping timer");
        	timer.cancel();
        }
    }
    
    @Override
    public void onWebSocketBinary(byte[] payload, int offset, int len) {
    	super.onWebSocketBinary(payload, offset, len);
    	String string = new String(payload);
    	logger.info("Server socket: Received BIN message: with length " + len + " [" + string + "]");
        if (string.equalsIgnoreCase("start") && session != null) {
        	logger.info("Starting timer");
        	timer.schedule(task, 10l, 1500l);
        }
        if (string.equalsIgnoreCase("stop") && session != null) {
        	logger.info("Stopping timer");
        	timer.cancel();
        }
    }
    
    @Override
    public void onWebSocketClose(int statusCode, String reason)  {
        super.onWebSocketClose(statusCode,reason);
        if (session.get() != null) {
        	session.get().close();
        }
        session.set(null);
        logger.info("Server socket Closed: [" + statusCode + "] " + reason);
    }
    
    @Override
    public void onWebSocketError(Throwable cause) {
        super.onWebSocketError(cause);
        logger.log(Level.SEVERE, "Server socket error", cause);
    }
}